//+-------------------------------------------------------------------+
//|                                       Relative Momentum Index.mq4 |
//|                              Original indicator by davinviproject |
//|                  Modified and Corrected by Linuxser for forex TSD |
//|                                                                   |
//| Relative Momentum Index (RMI)                                     |
//| Overview                                                          |
//| The Relative Momentum Index (RMI) was developed by Roger Altman.  |
//| It was first introduced in the February 1993 issue of Technical   |
//| Analysis of Stocks & Commodities magazine.                        |
//| The RMI is a variation of the RSI indicator. The RMI counts up and|
//| down days from the close relative to the close x-days ago (where x|
//| is not limited to 1 as is required by the RSI) instead of counting|
//| up and down days from close to close as the RSI does.             |
//|                                                                   |
//| Note that an RMI with parameters of C, 14, 1 is equivalent to a 14|
//| period RSI of the Close price. This is because the momentum       |
//|parameter is calculating only a 1-day price change (which the RSI  |
//|does by (default).                                                 |
//|As the momentum periods are increased the RMI fluctuations become  |
//|smoother.                                                          |
//| Since the RMI is an oscillator it exhibits the same strengths and |
//| weaknesses of other overbought / oversold indicators.             |
//|During strong trending markets it is likely that the RMI will      | 
//|remain at overbought or oversold levels for an extended period of  |
//|time.                                                              |  
//|However, during non-trending markets the RMI tends to oscillate more 
//|predictably between an overbought level of 70 to 90 and an oversold|
//|level of 10 to 30.                                                 |
//+-------------------------------------------------------------------+
#property copyright ""
#property link ""

#property indicator_separate_window
#property indicator_buffers 3
#property indicator_color1 LimeGreen
#property indicator_color2 DarkOrange
#property indicator_color3 DarkOrange
#property indicator_width1 2
#property indicator_width2 2
#property indicator_width3 2
#property indicator_minimum 0
#property indicator_maximum 100

//---- input parameters
extern int RMIPeriod=14;
extern int MomPeriod=5;
//---- buffers
double RMIBuffer[];
double RMIBufferda[];
double RMIBufferdb[];
double PosBuffer[];
double NegBuffer[];
double slope[];
//+------------------------------------------------------------------+
//| Custom indicator initialization function |
//+------------------------------------------------------------------+
int init()
{
//---- indicators
string short_name;
//---- 2 additional buffers are used for counting.
IndicatorBuffers(6);
//---- indicator line
SetIndexBuffer(0,RMIBuffer);
SetIndexBuffer(1,RMIBufferda);
SetIndexBuffer(2,RMIBufferdb);
SetIndexBuffer(3,PosBuffer);
SetIndexBuffer(4,NegBuffer);
SetIndexBuffer(5,slope);
SetLevelValue(0,30);
SetLevelValue(1,70);

//---- name for DataWindow and indicator subwindow label
short_name="RMI("+RMIPeriod+","+MomPeriod+")";
IndicatorShortName(short_name);
SetIndexLabel(0,short_name);
//----
SetIndexDrawBegin(0,RMIPeriod);
//----
return(0);
}
//+------------------------------------------------------------------+
//| RMI - Relative Momentum Index |
//+------------------------------------------------------------------+
int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
           int limit=MathMin(Bars-counted_bars,Bars-2);

   //
   //
   //
   //
   //
   
   if (slope[limit]==-1) CleanPoint(limit,RMIBufferda,RMIBufferdb);
   for (int i=limit; i>=0; i--) 
   {
      double sumn=0.0;
      double sump=0.0;
      if(i==Bars-RMIPeriod-1)
      {
         int k=Bars-2;
         //---- initial accumulation
         while(k>=i)
         {
            double rel=Close[k]-Close[k+MomPeriod];
            if(rel>0) 
                  sump+=rel;
            else  sumn-=rel;
            k--;
         }
         double positive=sump/RMIPeriod;
         double negative=sumn/RMIPeriod;
      }
      else
      {
      //---- simple moving average
         rel=Close[i]-Close[i+MomPeriod];
         if(rel>0) 
               sump=rel;
         else  sumn=-rel;
         positive=(PosBuffer[i+1]*(RMIPeriod-1)+sump)/RMIPeriod;
         negative=(NegBuffer[i+1]*(RMIPeriod-1)+sumn)/RMIPeriod;
      }
      PosBuffer[i]=positive;
      NegBuffer[i]=negative;
      if(negative==0.0) 
           RMIBuffer[i]=0.0;
      else RMIBuffer[i]=100.0*positive/(positive+negative);
           RMIBufferda[i] = EMPTY_VALUE;
           RMIBufferdb[i] = EMPTY_VALUE;
           slope[i]       = slope[i+1];
              if (RMIBuffer[i]>RMIBuffer[i+1]) slope[i] =  1;
              if (RMIBuffer[i]<RMIBuffer[i+1]) slope[i] = -1;
              if (slope[i]==-1) PlotPoint(i,RMIBufferda,RMIBufferdb,RMIBuffer);
   }
   return(0);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

void CleanPoint(int i,double& first[],double& second[])
{
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}

//
//
//
//
//

void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (first[i+1] == EMPTY_VALUE)
      {
         if (first[i+2] == EMPTY_VALUE) {
                first[i]   = from[i];
                first[i+1] = from[i+1];
                second[i]  = EMPTY_VALUE;
            }
         else {
                second[i]   =  from[i];
                second[i+1] =  from[i+1];
                first[i]    = EMPTY_VALUE;
            }
      }
   else
      {
         first[i]  = from[i];
         second[i] = EMPTY_VALUE;
      }
}